//---------------------------------------------------------------------------
// File: TMainForm.cpp
//
//  TDx_Sound_Library
//  Example3 - Playing Multiple Sounds
//
// Date: 21st June, 2002
// Author: Darren John Dwyer
// Copyright: (c) 2002 Darren John Dwyer, All Rights Reserved.
//
// Description:
//
// This example shows how to play multiple sound effects simultaneously
// simply by dropping different components onto a TForm.
//
// Note: you can actually use 1 TDSBufferDesc component if you prefer.
//
//---------------------------------------------------------------------------
#include <vcl.h>
#pragma hdrstop

#include "TMainForm.h"
//---------------------------------------------------------------------------
#pragma link "TDx_Sound"
#pragma link "TDx_SoundBuffer"
#pragma link "TDSCaps"
#pragma link "TDSBufferDesc"
#pragma link "TDx_Sound_Library_Install"
#pragma resource "*.dfm"
//---------------------------------------------------------------------------
TMainForm *MainForm;
//---------------------------------------------------------------------------
AnsiString BoolToStr( bool value )
{
return (value) ? "true" : "false";
}
//---------------------------------------------------------------------------
AnsiString HexToStr( unsigned int hexvalue )
{
char* hexchars[16] = {"0","1","2","3","4","5","6","7","8","9","A","B","C","D","E","F"};

AnsiString string = "0x";
int value = 0x10000000;

for (int i=0;i<8;i++)
    {
    string += hexchars[hexvalue/value];
    hexvalue -= value * (hexvalue/value);
    value = value >> 4;
    }
return string;
}
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
    : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormCreate(TObject *Sender)
{
KnownDevices = new TStringList();
KnownModules = new TStringList();
KnownGuids = new TList();

DevicesListBox->Items->Clear();
if (Dx_Sound1->DSEnumerate( NULL ))
    if (KnownDevices->Count>0)
        {
        DevicesListBox->ItemIndex = 0;
        Dx_Sound1->Create( (GUID*) KnownGuids->Items[DevicesListBox->ItemIndex] );
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormDestroy(TObject *Sender)
{
if (Dx_Sound1->Created) Dx_Sound1->Destroy();

if (KnownGuids) delete KnownGuids;
if (KnownModules) delete KnownModules;
if (KnownDevices) delete KnownDevices;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::DevicesListBoxClick(TObject *Sender)
{
if (Dx_Sound1->Created) Dx_Sound1->Destroy();
NotCreatedLabel->Visible = !Dx_Sound1->Create( (GUID*) KnownGuids->Items[DevicesListBox->ItemIndex] );
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Destroy(TObject *Sender)
{
if (Dx_SoundBuffer4->Created) Dx_SoundBuffer4->Destroy();
if (Dx_SoundBuffer3->Created) Dx_SoundBuffer3->Destroy();
if (Dx_SoundBuffer2->Created) Dx_SoundBuffer2->Destroy();
if (Dx_SoundBuffer1->Created) Dx_SoundBuffer1->Destroy();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::PlayButtonClick(TObject *Sender)
{
// scan through all selected files
for (int i=0;i<FileListBox1->Items->Count;i++)
    if (FileListBox1->Selected[i])
        {
        // determine if the file is already created
        bool created = (FileListBox1->Items->Strings[i]==FileNames[0]) ||
                       (FileListBox1->Items->Strings[i]==FileNames[1]) ||
                       (FileListBox1->Items->Strings[i]==FileNames[2]) ||
                       (FileListBox1->Items->Strings[i]==FileNames[3]);

        if (!created)
            {
            // check all 4 buffers and create in first available...
            if (!Dx_SoundBuffer1->Created)
                if (Dx_SoundBuffer1->CreateFromFile( FileListBox1->Items->Strings[i], DSBufferDesc1, Dx_Sound1 ))
                    {
                    FileNames[0] = FileListBox1->Items->Strings[i];
                    if (LoopingCheckBox->Checked) Dx_SoundBuffer1->Play(0,DSBPLAY_LOOPING);
                    else Dx_SoundBuffer1->Play(0,0);
                    continue;
                    }

            if (!Dx_SoundBuffer2->Created)
                if (Dx_SoundBuffer2->CreateFromFile( FileListBox1->Items->Strings[i], DSBufferDesc2, Dx_Sound1 ))
                    {
                    FileNames[1] = FileListBox1->Items->Strings[i];
                    if (LoopingCheckBox->Checked) Dx_SoundBuffer2->Play(0,DSBPLAY_LOOPING);
                    else Dx_SoundBuffer2->Play(0,0);
                    continue;
                    }

            if (!Dx_SoundBuffer3->Created)
                if (Dx_SoundBuffer3->CreateFromFile( FileListBox1->Items->Strings[i], DSBufferDesc3, Dx_Sound1 ))
                    {
                    FileNames[2] = FileListBox1->Items->Strings[i];
                    if (LoopingCheckBox->Checked) Dx_SoundBuffer3->Play(0,DSBPLAY_LOOPING);
                    else Dx_SoundBuffer3->Play(0,0);
                    continue;
                    }

            if (!Dx_SoundBuffer4->Created)
                if (Dx_SoundBuffer4->CreateFromFile( FileListBox1->Items->Strings[i], DSBufferDesc4, Dx_Sound1 ))
                    {
                    FileNames[3] = FileListBox1->Items->Strings[i];
                    if (LoopingCheckBox->Checked) Dx_SoundBuffer4->Play(0,DSBPLAY_LOOPING);
                    else Dx_SoundBuffer4->Play(0,0);
                    }
            }
         }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::StopButtonClick(TObject *Sender)
{
for (int i=0;i<FileListBox1->Items->Count;i++)
    if (FileListBox1->Selected[i])
        {
        if (FileListBox1->Items->Strings[i] == FileNames[0]) Dx_SoundBuffer1->Destroy();
        if (FileListBox1->Items->Strings[i] == FileNames[1]) Dx_SoundBuffer2->Destroy();
        if (FileListBox1->Items->Strings[i] == FileNames[2]) Dx_SoundBuffer3->Destroy();
        if (FileListBox1->Items->Strings[i] == FileNames[3]) Dx_SoundBuffer4->Destroy();
        }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FileListBox1DblClick(TObject *Sender)
{
PlayButtonClick(this);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Timer1Timer(TObject *Sender)
{
if (!Dx_SoundBuffer1->Playing) Dx_SoundBuffer1->Destroy();
if (!Dx_SoundBuffer2->Playing) Dx_SoundBuffer2->Destroy();
if (!Dx_SoundBuffer3->Playing) Dx_SoundBuffer3->Destroy();
if (!Dx_SoundBuffer4->Playing) Dx_SoundBuffer4->Destroy();

NameMemo->Clear();
SizeMemo->Clear();
StatusMemo->Clear();
PositionMemo->Clear();

// display buffer1 status
if (Dx_SoundBuffer1->Created)
    {
    dword buffer_size = DSBufferDesc1->BufferBytes;

    dword current_play_pos;
    dword current_write_pos;
    Dx_SoundBuffer1->GetCurrentPosition( &current_play_pos, &current_write_pos);

    NameMemo->Lines->Add( FileNames[0].SubString(1,14) );
    SizeMemo->Lines->Add( IntToStr(buffer_size) );
    PositionMemo->Lines->Add( IntToStr(current_play_pos) );

    Dx_SoundBuffer1->OpenStream( FileNames[0] );

    dword bytes_total = DSBufferDesc1->BufferBytes;
    dword bytes_remaining =  bytes_total - current_play_pos;
    dword format_tag = DSBufferDesc1->Format->wFormatTag;
    dword number_of_channels = DSBufferDesc1->Format->nChannels;
    dword samples_per_second = DSBufferDesc1->Format->nSamplesPerSec;
    dword average_bytes_per_second = DSBufferDesc1->Format->nAvgBytesPerSec;
    dword block_alignment = DSBufferDesc1->Format->nBlockAlign;
    dword bits_per_sample = DSBufferDesc1->Format->wBitsPerSample;

    float time_total = (float) buffer_size / (float) average_bytes_per_second;
    float percentage = (float) current_play_pos / (float) buffer_size;
    float other_percentage = 1.0 - percentage;

    float time_elapsed = percentage * time_total;
    float time_remaining =  other_percentage * time_total;

    TimeTotalLabel->Caption = "Time Total: "+FloatToStrF(time_total,ffFixed,2,2) + "s";
    TimeElapsedLabel->Caption = "Time Elapsed: "+FloatToStrF(time_elapsed,ffFixed,2,2) + "s";
    TimeRemainingLabel->Caption = "Time Remaining: "+FloatToStrF(time_remaining,ffFixed,2,2) + "s";

    TimeTotalLabel->Update();
    TimeElapsedLabel->Update();
    TimeRemainingLabel->Update();

    if (Dx_SoundBuffer1->Playing) StatusMemo->Lines->Add( "Playing..." ); else StatusMemo->Lines->Add( "Stopped..." );
    if (Dx_SoundBuffer1->StreamIsOpen) Dx_SoundBuffer1->CloseStream();
    }

// display buffer2 status
if (Dx_SoundBuffer2->Created)
    {
    dword temp_play_pos;
    dword temp_write_pos;
    Dx_SoundBuffer2->GetCurrentPosition( &temp_play_pos, &temp_write_pos);

    NameMemo->Lines->Add( FileNames[1].SubString(1,14) );
    SizeMemo->Lines->Add( IntToStr(DSBufferDesc2->BufferBytes) );
    PositionMemo->Lines->Add( IntToStr(temp_play_pos) );
    if (Dx_SoundBuffer2->Playing) StatusMemo->Lines->Add( "Playing..." ); else StatusMemo->Lines->Add( "Stopped..." );
    }

// display buffer3 status
if (Dx_SoundBuffer3->Created)
    {
    dword temp_play_pos;
    dword temp_write_pos;
    Dx_SoundBuffer3->GetCurrentPosition( &temp_play_pos, &temp_write_pos);

    NameMemo->Lines->Add( FileNames[2].SubString(1,14) );
    SizeMemo->Lines->Add( IntToStr(DSBufferDesc3->BufferBytes) );
    PositionMemo->Lines->Add( IntToStr(temp_play_pos) );
    if (Dx_SoundBuffer3->Playing) StatusMemo->Lines->Add( "Playing..." ); else StatusMemo->Lines->Add( "Stopped..." );
    }

// display buffer4 status
if (Dx_SoundBuffer4->Created)
    {
    dword temp_play_pos;
    dword temp_write_pos;
    Dx_SoundBuffer4->GetCurrentPosition( &temp_play_pos, &temp_write_pos);

    NameMemo->Lines->Add( FileNames[3].SubString(1,14) );
    SizeMemo->Lines->Add( IntToStr(DSBufferDesc4->BufferBytes) );
    PositionMemo->Lines->Add( IntToStr(temp_play_pos) );
    if (Dx_SoundBuffer4->Playing) StatusMemo->Lines->Add( "Playing..." ); else StatusMemo->Lines->Add( "Stopped..." );
    }
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitButtonClick(TObject *Sender)
{
Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer1Destroy(TObject *Sender)
{
FileNames[0] = "";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer2Destroy(TObject *Sender)
{
FileNames[1] = "";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer3Destroy(TObject *Sender)
{
FileNames[2] = "";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_SoundBuffer4Destroy(TObject *Sender)
{
FileNames[3] = "";
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1Create(TObject *Sender)
{
Dx_Sound1->SetCooperativeLevel(Application->Handle, DSSCL_PRIORITY);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Dx_Sound1DSEnumerate(TObject *Sender, GUID *Guid,
    AnsiString DriverDescription, AnsiString DriverModule, void *Context,
    bool &Finished)
{
if (KnownDevices && KnownModules && KnownGuids)
    {
    KnownDevices->Add( DriverDescription );
    KnownModules->Add( DriverModule );
    KnownGuids->Add( (void*) Guid );
    DevicesListBox->Items->Add( DriverDescription + ", ("+DriverModule+")" );
    }
else
    Finished = true;
}
//---------------------------------------------------------------------------




